// Implementation of the TLS class
/**
 
   O protocolo TLS envolve os seguintes passos:

   - O cliente envia uma mensagem "hello" contendo:
	- versão do protocolo
	- valor randomico
	- session ID* 
	- uma lista de algoritmos que ele suporta*
	- uma lista de métodos de compressão*
 
  - O servidor envia uma mensagem "hello" contendo:
	- versão do protocolo (igual ou menor que o valor enviado pelo cliente)
	- valor randomico
	- o algoritmo que ele suporta dentre os que estavam contido na lista de algoritmos
	- método de compressão que ele suporta dentre os que estavam contido na lista de métodos
	- session Id*

 *
 * - Exchange the necessary cryptographic parameters to allow the client and
 * server to agree on a premaster secret.
 *
 * - Exchange certificates and cryptographic information to allow the client
 * and server to authenticate themselves.
 *
 * - Generate a master secret from the premaster secret and exchanged random
 * values.
 *
 * - Provide security parameters to the record layer.
 *
 * - Allow the client and server to verify that their peer has calculated the
 * same security parameters and that the handshake occurred without tampering
 * by an attacker.
 *
 * Up to 4 different messages may be sent during a key exchange. The server
 * certificate, the server key exchange, the client certificate, and the
 * client key exchange.
 *
 * A typical handshake (from the client's perspective).
 *
 * 1. Client sends ClientHello.
 * 2. Client receives ServerHello.
 * 3. Client receives optional Certificate.
 * 4. Client receives optional ServerKeyExchange.
 * 5. Client receives ServerHelloDone.
 * 7. Client sends ClientKeyExchange.
 * 8. Client sends optional CertificateVerify.
 * 9. Client sends ChangeCipherSpec.
 * 10. Client sends Finished.
 * 11. Client receives ChangeCipherSpec.
 * 12. Client receives Finished.
 * 13. Client sends/receives application data.
 *
 * To reuse an existing session:
 *
 * 1. Client sends ClientHello with session ID for reuse.
 * 2. Client receives ServerHello with same session ID if reusing.
 * 3. Client receives ChangeCipherSpec message if reusing.
 * 4. Client receives Finished.
 * 5. Client sends ChangeCipherSpec.
 * 6. Client sends Finished.
 *
 */

#include "TLS.h"
//#include "tls/aes/aes.h"


//LINUX
#include "tls/arch/linux/ClientSocket.h"
#include "tls/arch/linux/SocketException.h"
#include <iostream>
#include <vector>

//#include <string>
#include <stdio.h>
#include <string.h>
#include <cstdlib>
#include <ctype.h>

//unix gmt_time
#include<ctime>

void printSeparator(int n){
	printf("\n\n"); 
	for(int i= 0;i<n;i++)
		printf("‖ ‖ ‖");
	printf("\n\n"); 
}


/*-- 
Método Responsável em imprimir uma sequência de bytes de maneira
padronizada. Ex: 
 >DDCCBBAA 43000000 53000000 79000000< ....C...S...y... 0x00000000 => 0x10BFE270
 >A2000000 63000000 9E000000 AA00003C< ....c..........< 0x00000010 => 0x10BFE280
*/

void hexdump(void *pAddressIn, long  lSize)
{

 
 printSeparator(25);
 	
 char szBuf[100];
 long lIndent = 1;
 long lOutLen, lIndex, lIndex2, lOutLen2;
 long lRelPos;
 struct { char *pData; unsigned long lSize; } buf;
 unsigned char *pTmp,ucTmp;
 unsigned char *pAddress = (unsigned char *)pAddressIn;

   buf.pData   = (char *)pAddress;
   buf.lSize   = lSize;
   int pAddressInTeste = (int)pAddressIn;
   while (buf.lSize > 0)
   {
      pTmp     = (unsigned char *)buf.pData;
      lOutLen  = (int)buf.lSize;
      if (lOutLen > 16)
          lOutLen = 16;

      // create a 64-character formatted output line:
      sprintf(szBuf, " >                            "
                     "                      "
                     "    0x%08lX => 0x%08lX", pTmp-pAddress, pTmp+pAddressInTeste);
      lOutLen2 = lOutLen;

      for(lIndex = 1+lIndent, lIndex2 = 53-15+lIndent, lRelPos = 0;
          lOutLen2;
          lOutLen2--, lIndex += 2, lIndex2++
         )
      {
         ucTmp = *pTmp++;

         sprintf(szBuf + lIndex, "%02X ", (unsigned short)ucTmp);
         if(!isprint(ucTmp))  ucTmp = '.'; // nonprintable char
         szBuf[lIndex2] = ucTmp;

         if (!(++lRelPos & 3))     // extra blank after 4 bytes
         {  lIndex++; szBuf[lIndex+2] = ' '; }
      }

      if (!(lRelPos & 3)) lIndex--;

      szBuf[lIndex  ]   = '<';
      szBuf[lIndex+1]   = ' ';

      printf("%s\n", szBuf);

      buf.pData   += lOutLen;
      buf.lSize   -= lOutLen;
   }


 printSeparator(25);

}


//array = {0xAB,0xFF}
//size: 1 OU 2 Ou 3 ou 4
unsigned int makeArrayCharToInt( char* array, int size )
{
   
   int i=0;
   unsigned arrToInt=0;
    for(i=0;i<size;i++)
        arrToInt = (arrToInt<<8) | (0XFF&array[i]);

   return arrToInt;
}

int reverseInt(int i){
    
    unsigned char c1, c2, c3, c4;

    if (is_bigendian()) {
        return i;
    } else {
        c1 = i & 255;
        c2 = (i >> 8) & 255;
        c3 = (i >> 16) & 255;
        c4 = (i >> 24) & 255;

        return ((int)c1 << 24) + ((int)c2 << 16) + ((int)c3 << 8) + c4;
    }

}



int RecordLayer::montarMsg( char** dt, int* size){

	printf("RecordLayer::montarMsg:\n");
	//const char* x = static_cast<const char*>(static_cast<void*>(this));
	*(dt) = static_cast<char*>(static_cast<void*>(this));
	*(size) = 5;
	return 0;
}

void RecordLayer::recuperarMsg(const char* data){

	printf("\n-----------------RecordLayer::recuperarMsg:\n");

	this->_content_type = data[0];
	this->_version_major = data[1];
 	this->_version_minor = data[2];

	char* dataSize = new char[2];
	dataSize[0] = data[3];dataSize[1] = data[4];
	int size = makeArrayCharToInt(dataSize, 2);
	this->_length = size;
	delete dataSize; 	


	printf("content_type: %X\n",_content_type);
	printf("version_major: %X\n",_version_major);
	printf("version_minor: %X\n",_version_minor);

	printf("length: %d\n",_length);
	printf("length[0][1]: [%X] [%X]\n",(0XFF&data[3]),(0XFF&data[4]));
	printf("\n-----------------\n");	
}


int HandshakeProtocol::montarMsg(char** dt, int* size){
	
    printf("HandshakeProtocol::montarMsg:\n");
    char * data = new char[4];
    data[0] = this->messageType;
    data[1] = this->handshakeLength[0];
    data[2] = this->handshakeLength[1];
    data[3] = this->handshakeLength[2];

    *(dt) = data;
    *(size) = 4;		
}

void HandshakeProtocol::recuperarMsg(const char* data){
   
   printf("\n-----------------HandshakeProtocol::recuperarMsg:\n");
   this->messageType = data[0];
   this->handshakeLength[0] = data[1];
   this->handshakeLength[1] = data[2];
   this->handshakeLength[2] = data[3];
   

	printf("message_type: %X\n",this->messageType);
	printf("hand_shake_lenght: %X\n",this->handshakeLength[0]);
	printf("hand_shake_lenght: %X\n",this->handshakeLength[1]);
	printf("hand_shake_lenght: %X\n",this->handshakeLength[2]);
	printf("\n-----------------\n");
    //return novo;
}

void Certificate::recuperarMsg(const char* data){
   
   printf("Cerificate::recuperarMsg:\n");
   this->content_lenght[0] = data[0];
   this->content_lenght[1] = data[1];
   this->content_lenght[2] = data[2];


	printf("hand_shake_lenght: %X\n",this->content_lenght[0]);
	printf("hand_shake_lenght: %X\n",this->content_lenght[1]);
	printf("hand_shake_lenght: %X\n",this->content_lenght[2]);

    //return novo;
}

ChangeCipherSpec* ChangeCipherSpec::recuperarMsg(const char* msg, u16 len){

ChangeCipherSpec* novo = new ChangeCipherSpec();

   novo->change_cipher_spec = msg[0];

    return novo;


}

int ClientHello::montarMsg(char** dt, int* size){

    printf("\n-----------------ClientHello::montarMsg:\n");

    char * data = new char[41];
    void* dataP = data;
     
    int gmt_unix_time_big_endian = reverseInt(this->gmt_unix_time);
    int random_bytes_big_endian[7];
    
    for(int i=0;i<7;i++)
       random_bytes_big_endian[i] = reverseInt(this->random_bytes[i]);
    
    data[0] = this->_version_major;
    data[1] = this->_version_minor;	    
	
    memcpy(dataP+2, &(gmt_unix_time_big_endian), 4); // gmt_unix_time = 4 bytes
    memcpy(dataP+6, random_bytes_big_endian , 28);//random_bytes = 28 bytes
    
    data[34] = 0x0;   // session_id_lenght
    data[35] = 0x00;  // method_compression_lenght		
    data[36] = 0x02;  //cypher_lenght	
    data[37] = this->cipher_suite0;
    data[38] = this->cipher_suite1;
    data[39] = 0x01;// method_compression_lenght
    data[40] = 0x0;// method_compression (NULL)	
  
    //const char* x = (const char*) data;//dataFinal;

    *(dt) = data;
    *(size) = 41;	
    printf("\n-----------------\n");
    return 0;	

}


void ServerHello::recuperarMsg(const char* data){
    //hexdump(const_cast<char*>(data), 400);

   printf("\n-----------------ServerHello::recuperarMsg:\n");
   //this->version_major = data[0];
   //this->version_minor = data[1];

	//NAO FUNCIONA, WHY?
	//char gmt[4];gmt[0] = data[2];gmt[1] = data[3];gmt[2] = data[4];gmt[3] = data[5];
	//this->gmt_unix_time = makeArrayCharToInt(gmt, 4);

	this->gmt_unix_time[0] = data[2];
	this->gmt_unix_time[1] = data[3];
	this->gmt_unix_time[2] = data[4];
	this->gmt_unix_time[3] = data[5];

	for(int i=0;i<28;i++){
		this->random_bytes[i] = data[6+i];
	}

	
	this->session_id_lenght = data[34];

	for(int i=0;i<this->session_id_lenght;i++)
		this->session_id[i] = data[35+i];

	this->cipher_suite[0] = data[(int)this->session_id_lenght +35];
	this->cipher_suite[1] = data[(int)this->session_id_lenght +36];
	this->compression_method = data[(int)this->session_id_lenght +37];
	
	/*
	printf("gmt_unix_time %X\n",this->gmt_unix_time[0]);
	printf("gmt_unix_time %X\n",this->gmt_unix_time[1]);
	printf("gmt_unix_time %X\n",this->gmt_unix_time[2]);
	printf("gmt_unix_time %X\n",this->gmt_unix_time[3]);
	for(int i=0;i<28;i++)
		printf("random: %X\n",this->random_bytes[i]);
	printf("session id leghth %X\n",this->session_id_lenght);
	for(int i=0;i<this->session_id_lenght;i++)
		printf("session_id: %X\n",this->session_id[i]);
	printf("cipher_suite[0] %X\n",this->cipher_suite[0]);
	printf("cipher_suite[1] %X\n",this->cipher_suite[1]);
	printf("compression_method %X\n",this->compression_method);*/
    //return novo;
       printf("\n-----------------\n");	
}

int Finished::montarMsg(char** dt, int* size){

	char * data = new char[5];

	data[0] = *verify_data;
	memcpy((void*)data[1], (void*)verify_data_size, 4);

	*(dt) = data;
	*(size) = 5;
	//const char* x = (const char*) data;

     return 0;


}

Finished* Finished::recuperarMsg(const char* msg, u16 len){

	Finished* novo = new Finished();
	novo->verify_data = (char*)msg[0];
	memcpy((void*)novo->verify_data_size, (void*)msg[1], 4);
	return novo;


}

int ApplicationData::montarMsg(char** dt, int* size){

	char * data = new char[strlen(msg)];//checar se pega o tamanho correto da msg

	memcpy(data, msg, strlen(msg));

	//const char* x = (const char*) data;
	*(dt) = data;
	*(size) = 5;
     return 0;


}

ApplicationData* ApplicationData::recuperarMsg(const char* msg, u16 len){

	ApplicationData* novo = new ApplicationData();
	memcpy(novo->msg, (char*)msg, len);//checar se pega o tam correto

	return novo;
}


TLS::TLS (char* h)
{
	this->host = h;
	this->_state = START; 

	//ARCH DEPENDENT
	this->_gmt_unix_time = std::time(0);
}


TLS::~TLS ()

{       //ARCH linux
	delete (this->client_socket);
}


void TLS::teste_aes(){

/*
	int i;
	
	const unsigned char master_key[16] = {
		0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
		0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
	};
	const unsigned char text[16] = {
		0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
		0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff,
	};
	
	unsigned char encrypted[16], decrypted[16];
	
	ns_AES::AES_KEY key;
	ns_AES::AES *aes = new ns_AES::AES();
	printf("plaintext:\n");
	for (i = 0; i < 16; i++) {
		printf("%02x ", (unsigned int)text[i]);
	}
	printf("\n\n");
	
	printf("encrypted:\n");
	aes->AES_set_encrypt_key(master_key, 128, &key); // 128 for 128-bit version AES
	aes->AES_encrypt(text, encrypted, &key);
	for (i = 0; i < 16; i++) {
		printf("%02x ", (unsigned int)encrypted[i]);
	}
	printf("\n\n");
	
	printf("decrypted:\n");
	aes->AES_set_decrypt_key(master_key, 128, &key);
	aes->AES_decrypt(encrypted, decrypted, &key);
	for (i = 0; i < 16; i++) {
		printf("%02x ", (unsigned int)decrypted[i]);
	}
	printf("\n");
*/

}


//ARCH LINUX

void TLS::tcp_conectar()
{

  try
    {
       std::string str_host_name = this->host;
       //std::string str_host = "moodle.ufsc.br";//moodle.ufsc.br => 150.162.1.162";//"74.125.234.181";//"127.0.0.1";

       this->client_socket =  new ClientSocket(str_host_name, 443);//TLS para HTTPS, TODO: deixar porta genérica//5001);
     

    }
  catch ( SocketException& e )
    {
      std::cout << "Exception was caught:" << e.description() << "\n";
    }

}

void TLS::tcp_send(const char* in, int s){
      
      std::string str_in(in, s);	

      printf("\nSending....");
      hexdump(const_cast<char*>(in), s);
	
      try{
	  *(client_socket) << str_in;
	}catch ( SocketException& ) {}

}

char* TLS::tcp_receive(int* size){
      //std::string reply;
      std::vector<char> reply;

      try{
	  *(client_socket) >> reply;
      }catch ( SocketException& ) {}

      printf("\n Receiving....(%d) bytes",(int) reply.size() );
      int lenght = reply.size();
      char* c = new char[lenght];
      
      int length;
      for(int i =0; i < (int)reply.size();i++){
 	c[i] = reply[i];
      }

      //hexdump(const_cast<char*>(reply.c_str()), reply.size());
      hexdump(c, lenght);
      *(size) = (int) reply.size();	
      return c;	

}

/*
char* TLS::tcp_send_receive(const char* in, int s)
{
      
      std::string str_in(in, s);	
      std::string reply;
    
      printf("\nSending....");
      hexdump(const_cast<char*>(in), s);
 
	
      try
	{
	  *(client_socket) << str_in;
	  *(client_socket) >> reply;
	}
      catch ( SocketException& ) {}

      printf("\n Receiving....");
      hexdump(const_cast<char*>(reply.c_str()), reply.size());

      const char *current = reply.c_str();
      char *x;
      x = const_cast<char*>(current);
      return x;	
}*/


char* TLS::send_receive(char* message, int s)
{

 
      printf("\nA camada acima (HTTP/SMTP/FTP) quer enviar a seguinte mensagem:");
      hexdump(const_cast<char*>(message), s);

      if(this->state() == START){

	this->_START();

      }

      return message;
	
}


// END ARCH LINUX
/*
TODO: ficaria mais facil usar struct para "serializar" e des-serializar
namespace STRUCTS{


       struct ProtocolVersion {
           u8 major, minor;
       };

       struct Record_Layer {
           u8 type;
           ProtocolVersion version;
           u16 length;
       };

       struct HandShake{
          u8 msg_type;    
          u8 length[3];     
      };

      struct Random {
        u32 gmt_unix_time;
        u8 random_bytes[28];
      };

      struct SessionID{
        u8 lenght;
        u8 content[]; 
      };
	
     struct CipherSuite{
        u8 lenght;// = 0x01;  // quantide de cypher_suite implementadas. 1 par de bytes => (1 cypher suite ID)
        u8 content[];//2 = {0x00, 0x2F }; // TLS_RSA_WITH_AES_128_CBC_SHA     = { 0x00, 0x2F };
      };

      struct CompressionMethod{
        u8 lenght;// = 0x01;  // quantide de métodos 
        u8 content;// = 0x00; // method null 
      };	

    
      struct ClientHello{
          ProtocolVersion client_version;
          Random random;
          SessionID session_id;
          CipherSuite cipher_suites;
          CompressionMethod compression_methods;
      };

}*/


namespace TLS_RECORD
{
	namespace PROTOCOL
	{
		enum
		{
		        CHANGE_CIPHER_SPEC = 0x14,
		        ALERT =  0x15,
		        HANDSHAKE = 0x16,
			APPLICATION = 0x17
		};
	}

	//Message type that identifies the Handshake message type.
	namespace HANDSHAKE
	{
		enum
		{
		        HELLO_REQUEST = 0,
    		        CLIENT_HELLO = 1,
    	 	        SERVER_HELLO = 2,
    	 	        CERTIFICATE = 11,
     			SERVER_KEY_EXCHANGE = 12,
	     		CERTIFICATE_REQUEST = 13,
	     		SERVER_HELLO_DONE = 14,
	     		CERTIFICATE_VERIFY = 15,
	     		CLIENT_KEY_EXCHANGE = 16,
	     		FINISHED = 20
		};
	}

	namespace VERSION
	{
		namespace MAJOR
		{
			enum
			{
				SSL_3_0 = 3,
				TLS_1_0 = 3,
				TLS_1_1 = 3,
				TLS_1_2 = 3
			};
		}

		namespace MINOR
		{
			enum
			{
				SSL_3_0 = 0,
				TLS_1_0 = 1,
				TLS_1_1 = 2,
				TLS_1_2 = 3
			};
		}
	}

	namespace ALERT
	{
		  enum
		  {
			  CLOSE_NOTIFY= 0,
			  UNEXPECTED_MESSAGE= 10,
			  BAD_RECORD_MAC= 20,
			  DECRYPTION_FAILED= 21,
			  RECORD_OVERFLOW= 22,
			  DECOMPRESSION_FAILURE= 30,
			  HANDSHAKE_FAILURE= 40,
			  BAD_CERTIFICATE= 42,
			  UNSUPPORTED_CERTIFICATE= 43,
			  CERTIFICATE_REVOKED= 44,
			  CERTIFICATE_EXPIRED= 45,
			  CERTIFICATE_UNKNOWN= 46,
			  ILLEGAL_PARAMETER= 47,
			  UNKNOWN_CA= 48,
			  ACCESS_DENIED= 49,
			  DECODE_ERROR= 50,
			  DECRYPT_ERROR= 51,
			  EXPORT_RESTRICTION= 60,
			  PROTOCOL_VERSION= 70,
			  INSUFFICIENT_SECURITY= 71,
			  INTERNAL_ERROR= 80,
			  USER_CANCELED= 90,
			  NO_RENEGOTIATION= 100
		};
	}

}



// Métodos para modelar os estados


/* "Estado" responsável em montar e enviar a mensagem ClientHello com 
  a versão do protocolo (TLS 1.0), um valor randômico (struct com 
  gmt_unix_time (4 bytes) e random de 28 bytes), session_id,
  cipher_suite suportada e método de compressão (null).
  — a única cipher suite que suportamos é a 0x00,0x3C (AES_RSA_SHA256)*/
void TLS::_START() {

	 printf("\nEstado (START)...\n");
	
	//montar record
	ClientHello* msgClientHello = new ClientHello();
	msgClientHello->_version_major = TLS_RECORD::VERSION::MAJOR::TLS_1_0;
	msgClientHello->_version_minor = TLS_RECORD::VERSION::MINOR::TLS_1_0;


	//dados referente ao hello message
	msgClientHello->gmt_unix_time = this->_gmt_unix_time;//0XAABBCCDD; //
	srand(time(NULL));
	for(int i = 0; i < 7; i++){
		msgClientHello->random_bytes[i] = rand() % 0xFFFFFFFF;

		//armazenando na classe TLS o valor randomico que o cliente vai enviar		
		this->randomClient[i+1] = msgClientHello->random_bytes[i];
	}
	  
	msgClientHello->cipher_suite0 = 0x00;
	msgClientHello->cipher_suite1 = 0x2F; //CipherSuite TLS_RSA_WITH_AES_128_CBC_SHA     = { 0x00, 0x2F };
	char* streamBytes_ch;int size_client_hello;
	msgClientHello->montarMsg(&streamBytes_ch, &size_client_hello);
       
	//dados referente  ao handshake layer
	msgClientHello->messageType = TLS_RECORD::HANDSHAKE::CLIENT_HELLO;
	msgClientHello->handshakeLength[0] = (size_client_hello >> 16) & 0xFF ;
	msgClientHello->handshakeLength[1] = (size_client_hello >> 8) & 0xFF;
	msgClientHello->handshakeLength[2] = (size_client_hello) & 0xFF;
	char* streamBytes_hs;int size_handshake;
	msgClientHello->HandshakeProtocol::montarMsg(&streamBytes_hs, &size_handshake);
	
   
    	//dados do record layer
	msgClientHello->_content_type = TLS_RECORD::PROTOCOL::HANDSHAKE;
	msgClientHello->_length = size_client_hello + size_handshake;
	int size_recordlayer;char* streamBytes_rl;
        msgClientHello->RecordLayer::montarMsg(&streamBytes_rl, &size_recordlayer);

	//GERANDO STREAM RECORD_LAYER + HANDSHAKE + CLIENTHELLO
	int sum_of_size = size_client_hello + size_handshake + size_recordlayer;
	char* streamBytes_final = new char[sum_of_size];
    	memcpy(streamBytes_final, streamBytes_rl ,size_recordlayer); 
    	memcpy(streamBytes_final + size_recordlayer, streamBytes_hs ,size_handshake); 
	memcpy(streamBytes_final + size_recordlayer + size_handshake, streamBytes_ch , size_client_hello);

	randomClient[0] = msgClientHello->gmt_unix_time;
	

	//finished = new char[93];
	/*copiar a msg pro finished
	    memcpy((void*)finished[0], (void*)message->gmt_unix_time, 4);
	    memcpy((void*)finished[4], (void*)randomClient, 28);
	    finished[32] = message->cipher_suite0;
	    finished[33] = message->cipher_suite1;
	    finished[34] = message->messageType;
	    finished[35] = message->handshakeLength[0];
	    finished[36] = message->handshakeLength[1];
	    finished[37] = message->handshakeLength[2];
	*/

	//transformar em const char*
    	//delete dataRecord;
    	//delete dataHandShake;
    	//delete data;
	
	//enviar tcp;
        this->tcp_send(streamBytes_final,sum_of_size);

	setState(HELLO_SENT);
	_HELLO_SENT();//DESNCESSARIO COM HANDLER*/
}

/* Neste estado já foi enviado o ClientHello, e na verdade esse seria um
   estado desnecessário, mas fica mais compreensível, dá uma ideia de
   continuidade. Podemos aproveitar para gerar o Premaster Secret aqui
   — pms = versão TLS (3,3) + 46 random bytes (random_client + random_server).
*/
void TLS::_HELLO_SENT() {
	
	 printf("\nEstado (HELLO_SENT)...\n");

	//?????

	setState(KEY_EXCHANGE);
	_KEY_EXCHANGE();
}

void TLS::_KEY_EXCHANGE() {

/* Começa a complicar... Eu havia sugerido agrupar 3 estados em 1, mas 
  agora já penso se realmente foi uma boa solução.
   Neste ponto estamos esperando 3 mensagens: ServerHello, Certificate e
  ServerHelloDone. Será que agrupamos as 3 por fora e executamos o estado
  tendo as 3 mensagens separadas?
   Basicamente, o que acontece aqui é:
  - confere o ServerHello do servidor, pegando o random dele (que será
  usado futuramente);
  - verificar o certificado do servidor, i.e. se a data está ok, se
  corresponde ao site, etc;
  - pegar a chave pública do servidor do certificado x.509;
  - Enviar 2 mensagens:
    ClientKeyExchange (pms criptografado com a chave pública do servidor);
     - Aqui, geramos o master secret enquanto o servidor faz o mesmo:
	- P_hash(pms, "master secret" + random_cli + random_srv).
    ChangeCipherSpec (simplesmente 1 byte com valor 1);
*/
	 printf("\nEstado (KEY_EXCHANGE)...\n");
	int size = 0;
	char* retorno = this->tcp_receive(&size);
	int sizeTotalMensagem = size;
/*
	u16 sizePsk = 23;
	u16 sizePms = 48;
        pre_master_secret = computePreMasterSecret(psk, sizePsk);
	master_secret = computeMasterSecret(pre_master_secret, sizePms);
*/



	//remontar msg
	ServerHello* message = new ServerHello();
	message->RecordLayer::recuperarMsg(retorno);
	message->HandshakeProtocol::recuperarMsg(retorno+5);

	//verifica se mensagem é um ServerHello
	if(message->messageType == TLS_RECORD::HANDSHAKE::SERVER_HELLO){
		
		message->recuperarMsg(retorno+(9));//5 bytes RecordLayer + 4 bytes do HandshakeProtocol
		
		//obter versao e comparar com o client
		//obter cipher_suite e comparar com o cipher do client
		//printf("mvm %X\n",message->_version_major);
		//printf("mvm %X\n",message->_version_minor);
		//printf("mvm %X\n",message->cipher_suite[0]);
		//printf("mvm %X\n",message->cipher_suite[1]);
		
		if((message->_version_major != TLS_RECORD::VERSION::MAJOR::TLS_1_0) || (message->_version_minor != TLS_RECORD::VERSION::MINOR::TLS_1_0) || (message->cipher_suite[0] != 0x00) || (message->cipher_suite[1] != 0x2f)){
				
				printf("O servidor não suporta a versão 1.0 do TLS. Ou não suporta a cipher_suite 0x002f");

				setState(CLOSE);
				_CLOSE();//DESNCESSARIO COM HANDLER*/
		}else{
		
			//obter num random
			randomServer[0] = (message->gmt_unix_time[0] << 24) | (message->gmt_unix_time[1] << 16) | (message->gmt_unix_time[2] << 8) | (message->gmt_unix_time[3]);
			//printf("\nRandomServer (%X) =>", randomServer[0] );
			
			for(int i = 0; i < 7; i++)
				randomServer[i+1] = (message->random_bytes[0+(i*4)] << 24) | (message->random_bytes[1+(i*4)] << 16) | (message->random_bytes[2+(i*4)] << 8) | (message->random_bytes[3+(i*4)]);
			

			//for(int i = 0; i < 7; i++)
				//printf("\nRandomServer (%X) =>", randomServer[i+1] );

			//printf("\nBBBB");

			/*	//copiar finished
				memcpy((void*)finished[38], (void*)message->gmt_unix_time, 4);
			memcpy((void*)finished[42], (void*)randomServer, 28);
			finished[60] = message->cipher_suite0;
			finished[61] = message->cipher_suite1;
			finished[62] = message->handshakeLength[0];
			finished[63] = message->handshakeLength[1];
			finished[64] = message->handshakeLength[2];
				}
			}*/



			/*-- CERTIFICATE --*/

			int serverhello_length = (message->_length-4);//padding recordlayer + handshakeprotocol
			printf("\nServerhello_length(%d) =>", serverhello_length );
	
			printf("\nAinda há mais %d bytes na mensagem", sizeTotalMensagem - (serverhello_length+9) );////5 padding recordlayer +  4handshakeprotocol

			char* retorno2 = retorno+serverhello_length+9;

		   	printf("\nRetorno Restante....");
			hexdump(const_cast<char*>(retorno2), 500);

			if(sizeTotalMensagem > serverhello_length+9){
		
				Certificate* cert = new Certificate();
				cert->RecordLayer::recuperarMsg(retorno2); //padding recorlayer
				cert->HandshakeProtocol::recuperarMsg(retorno2+5);//padding handshakeProtocol
		
				printf("\nTipo mensagem....%d",cert->messageType);

				//verifica se mensagem é um Cerificate
				if(cert->messageType == TLS_RECORD::HANDSHAKE::CERTIFICATE){
			
			
					/* -- TODO -- */
					/*cert->recuperarMsg(retorno+(9));//5 bytes RecordLayer + 4 bytes do HandshakeProtocol
		
					char* dataSize = new char[3];
					dataSize[0] = cert->content_lenght[0];
					dataSize[1] = cert->content_lenght[1];
					dataSize[2] = cert->content_lenght[2];
					int size = makeArrayCharToInt(dataSize, 3);
					*/
				
					/**
					 * Called when a client receives a Certificate record.
		
					 * Meaning of this message:
					 *   The certificate type must be appropriate for the selected cipher suite's
					 *   key exchange algorithm, and is generally an X.509v3 certificate. It must
					 *   contain a key which matches the key exchange method, as follows. Unless
					 *   otherwise specified, the signing algorithm for the certificate must be
					 *   the same as the algorithm for the certificate key. Unless otherwise
					 *   specified, the public key may be of any length.
					 *
					 * opaque ASN.1Cert<1..2^24-1>;
					 * struct {
					 *   ASN.1Cert certificate_list<1..2^24-1>;
					 * } Certificate;*/
					//TODO
					//TODO
					//TODO
					//TODO
					//PARSER ASN.1Cert
					//TODO
				
					int certificate_length = (cert->_length-4);//padding recordlayer
					printf("\nCertificate_length(%d) =>", cert->_length);
	
					printf("\nAinda há mais %d bytes na mensagem", sizeTotalMensagem - (((serverhello_length+9) + (certificate_length+9))));////5 padding recordlayer +  4handshakeprotocol

					char* retorno3 = retorno2+certificate_length+9+166;

				   	printf("\nRetorno Restante....");
					hexdump(const_cast<char*>(retorno3), 9);
		

					//REstante Server_Hello_Done
					if(sizeTotalMensagem > ((serverhello_length+9)+(certificate_length+9))){
		
					HandshakeProtocol* server_hello_done = new HandshakeProtocol();
					server_hello_done->RecordLayer::recuperarMsg(retorno3); //padding recorlayer
					server_hello_done->HandshakeProtocol::recuperarMsg(retorno3+5);//padding handshakeProtocol
			
					//Classe ServerHello é vazia				

					printf("\nTipo mensagem....%X",server_hello_done->messageType);

					//verifica se mensagem é um Cerificate
					if(server_hello_done->messageType == TLS_RECORD::HANDSHAKE::SERVER_HELLO_DONE){
			
						//TUDO OK
						//ir para o proximo estado

						//VAMOS ENVIAR O CLIENT
						/*

							Mensagem Client Key Exchange:
							Como o cliente não enviará certificado no nosso caso, essa msg deve ser a PRIMEIRA do cliente depois de receber o ServerHelloDone;
							No caso do RSA, o servidor não precisa enviar o ServerKeyExchange, apenas em Diffie-Hellman;
							Basicamente, esta msg contém o premaster secret criptografado pela chave pública do servidor no caso do RSA, e os parâmetros necessários no caso de DH;
							O premaster secret é formado por: versão do cliente (que é 3,3 no caso do TLS 1.2 e é enviada no ClientHello.client_version) + aleatório de 46 bytes.
							Depois, do ClientKeyExchange, cliente e servidor vão gerar o master secret da seguinte forma:
							48 primeiros bytes do resultado da aplicação da PRF (Pseudo Randon Function) com os parâmetros pre_master_secret, a label "master secret", e a soma de random do cliente + random do servidor;
							OBS: Cliente e servidor geram seus números aleatórios independentes de 28 bytes para a negociação e trocam eles no Hello.
							A PRF é definida pela RFC assim: 
							P_hash(secret, seed) = HMAC_hash(secret, A(1) + seed) +
										     HMAC_hash(secret, A(2) + seed) +
										     HMAC_hash(secret, A(3) + seed) + ...
							A(0) = seed; A(1) = HMAC_hash(secret, A(i-1))
							Got it? No caso da master secret a PRF é chamada com (pre_master_secret, label + seed), e seed = random cliente + random servidor;
							HMAC_hash, como já disse, é SHA-256.
							A cipher suite que vamos utilizar, portanto, é TLS_RSA_WITH_AES_128_CBC_SHA256. A não ser que alguém tenha algo contra, huahuahua.						
						*/
						
						printf("Indo para o estado Finished ServerHelloDone!");

						setState(SEND_FINISHED);
						_SEND_FINISHED();
				
					}

			
					}else{


						printf("O servidor não enviou o ServerHelloDone!");

						setState(CLOSE);
						_CLOSE();//DESNCESSARIO COM HANDLER*/

					}
		
		
				}

					printf("O servidor não enviou o Certificado!");

					setState(CLOSE);
					_CLOSE();//DESNCESSARIO COM HANDLER*/
		
		
			}//fim do segundo retorno
				

		}//fim else (serverhello com dados compatíveis do clieente 

	}else{//fim if é server_hello
		printf("O servidor não enviou um ServerHello!");

		setState(CLOSE);
		_CLOSE();//DESNCESSARIO COM HANDLER*/
	}


}

void TLS::_SEND_FINISHED() {
/* Neste ponto, enviamos a mensagem Finished, que é montada ao longo do
  handshaking... Não estou certo se o jeito que o pessoal do outro semestre  fez está correto. Mas parece que sim.
   Algumas considerações sobre essa msg:
  - os dados vão após passarem pela PRF com a label "client finished";
  - é erro fatal se a Finished não é precedida pela ChangeCipherSpec.
*/

	printf("\nAA");
	setState(WAIT_FINISHED);
	_WAIT_FINISHED();
}

void TLS::_WAIT_FINISHED() {
/* Aguardamos um changeCipherSpec e o Finished do servidor. Comparamos o  
  Finished recebido com o enviado. Se for igual, ok! Tá pronto pra iniciar
  a transferência de dados. Caso contrário -> CLOSE. 
*/

	setState(READY);
	_READY();
}

void TLS::_READY() {


	setState(SEND_DATA);
	_SEND_DATA();
}


void TLS::_SEND_DATA() {

	setState(SEND_DATA);
	_SEND_DATA();
}

void TLS::_RECV_DATA() {

	setState(CLOSE);
	_CLOSE();
}

void TLS::_CLOSE() {

	
}


/*
u8* TLS::hashSHA(u8* secret, u16 size){
	u8* temp = new u8[64];
	for(int i = 0; i < 64; i++){
        temp[0] = 0;
	}
    return temp;
}

u8* TLS::prf(u8* secret, u16 secretSize, char* label, u8* seed){
    //return hashSHA(secret, secretSize , strcat(label, (const char *)seed));
    u8* temp = new u8[64];
	for(int i = 0; i < 64; i++){
        temp[i] = 0;
	}
    return temp;
}

u8* TLS::computePreMasterSecret(u8* psk, u16 length){
    u8* temp = new u8[48];
    temp[0] = 23;
    for(int i = 0; i < 23; i++){
        temp[i+1] = 0;
    }
    temp[24] = 23;
    for(int i = 0; i < 23; i++){
        temp[i+25] = psk[i];
    }
    return temp;
}

u8* TLS::computeMasterSecret(u8* pms, u16 length){
    u8* temp = new u8[48];

    char* label = new char[13];
	label = "master secret";
	char * x = (char*) randomClient;
	const char * y = (const char*) randomServer;
	strcat(x, y);
	u8* z = (u8*) x;
    temp = prf(pms, 48, label, z);

    return temp;
}*/
